/*******************************************************************************
 * Copyright 2007 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace GetFileForBuild
{
    class Program
    {
        static void Main(string[] args)
        {
            
         //Let's define some variables we will use with SOAP 
            string currProject = "Sample Project";
            string soapUser = "Administrator";
            string soapUserPassword = "";
            string defectSelected = System.Environment.GetEnvironmentVariable("DNUM");

            long theCookie = 0; ;
            //define the ttsoapcgi object
            ttsoapcgi ttSoap;
            //creating an instance of this object
            ttSoap = new ttsoapcgi();
            //set the URL of the ttsoapcgi object
            ttSoap.Url = "http://localhost:80/scripts/ttsoapcgi.exe";
            
            //Log In User
            try
            {
                theCookie = ttSoap.DatabaseLogon(currProject, soapUser, soapUserPassword);
                               
            }
            catch (Exception err)
            {
                Logger.log(err.Message);
            }
            
            //Use SOAP to check that the last event is in fact the event that should get the file
            try
            {
                //Get the number of the defect
                long num = Convert.ToInt64(defectSelected);
                //Define and instantiate the defect object
                CDefect pDefect = ttSoap.getDefect(theCookie, num, "", true);

                //We establish the index for where the event will be listed in the event list array of the defect 
                 
                int eventIndex = pDefect.eventlist.Length - 2;
                //Test to make sure this is the correct event, the event we are looking for did not take place.
                if (pDefect.eventlist[eventIndex].name == "Get Files For Build")
                {
                    //Get the number of scc files
                    string sccFileCount = System.Environment.GetEnvironmentVariable("SCC_CNT");
                    //before we change it to an int, set the int to 0
                    int intSCCFileCount = 0;
                    try
                    {
                        intSCCFileCount = Convert.ToInt16(sccFileCount);


                    }
                    catch(Exception fcc)
                    {
                        Logger.log(fcc.Message);
                    }
                    //if int is greater than zero, it means we did find a count of scc files AND were able to convert it
                    //from string to int.
                    if (intSCCFileCount > 0)
                    {
                        for (int i=1; i <= intSCCFileCount; i++)
                        {
                            string sccFile = System.Environment.GetEnvironmentVariable("SCC" + i + "_Name");
                            string fixRev = System.Environment.GetEnvironmentVariable("SCC" + i + "_VERS");
                            Logger.log("SCC File #" + i.ToString() + " Name: " + sccFile);
                            Logger.log("SCC File #" + i.ToString() + " Versions: -" + fixRev + "-" );
                    
                            string scmServer = "127.0.0.1:4900";
                            string scmUser = "Administrator";
                            string scmPassword = "";
                            string commandParams = "";
                            /* now let's parse the results
                            we know that the result will be in the format:
                            <branch name>::<repo path/filename>
                                                                 */

                            //first, we find the position of the first colon to get the branch name
                            int pos = sccFile.IndexOf(":");
                            string branchName = sccFile.Substring(0, pos);
                            //we will remove the branch name and colons from the rest of the string we have to parse
                            sccFile = sccFile.Substring(pos + 2);
                            //let's reset the pos variable so we can use it again
                            pos = 0;
                            //next, we find the last "/".  Anything before this will the repo and anything after will be the file name
                            pos = sccFile.LastIndexOf("/");
                            string repoName = sccFile.Substring(0, pos);
                            string fileName = sccFile.Substring(pos + 1);

                            /*
                            Now let's run the command to get the file from Surround SCM-
                               * Here is the syntax of the command:
                               * 
                               * sscm get Item1 [Item2 ...] [-bBranch] [-dLocalDir] [-e] [-f] [-h][-pRepository] [-lLabel|-sTimeStamp|-vVersion] [-i|-i-] [-j]
                                    [-q] [-r] [-tcurrent|-tmodify|-tcheckin] [-wprompt|-wreplace|-wskip][-aStateID] [-y[Username:Password|+]] [-zSCMServerAddr:PortNum] [+u]
                               * 
                               */
                            if (fixRev == null)
                            {
                                commandParams = "get " + "\"" + fileName + "\"" + " -b" + "\"" + branchName + "\"" + " -p" + "\"" + repoName + "\"" + " -y" + "\"" + scmUser + ":" + scmPassword + "\"" + " -z" + scmServer;
                            }
                            else
                            {
                                //if there is a 'fixed revision', then get that version - note that at this time this only works if the file has a single fixed revision
                                commandParams = "get " + "\"" + fileName + "\"" + " -b" + "\"" + branchName + "\"" + " -p" + "\"" + repoName + "\"" + " -v" + fixRev + " -y" + "\"" + scmUser + ":" + scmPassword + "\"" + " -z" + scmServer;
                            }
                            string sscmresult = sscmexe.sscmCMD(commandParams);

                            Logger.log(sscmresult);
                                                                           
                        }

                    }
                    
                }
                
                
            }
            catch (Exception fcc)
            {
                Logger.log(fcc.Message);
            }

            try
            {
                ttSoap.DatabaseLogoff(theCookie);


            }
            catch (Exception err)
            {
                Logger.log(err.Message);
            }
                
            
            
            
                       
            


        }
    }
}
